@@ -1,4 +1,11 @@
-import datetime
+"""
+==================
+Distributed Worker
+==================
+
+Custome RQ worker for running vivarium jobs.
+
+"""
 import json
 import math
 import os
@@ -15,7 +22,7 @@ from loguru import logger
 from rq import Queue, get_current_job
 from rq.job import JobStatus
 from rq.registry import FailedJobRegistry
-from rq.worker import StopRequested, Worker
+from rq.worker import Worker
 
 from vivarium_cluster_tools.psimulate import globals as vct_globals
 from vivarium_cluster_tools.vipin.perf_counters import CounterSnapshot
@@ -49,9 +56,9 @@ class ResilientWorker(Worker):
 
     def work(self, *args, **kwargs):
         worker_ = self.name
-        logger.remove()
         logging_directory = Path(os.environ['VIVARIUM_LOGGING_DIRECTORY'])
         logger.add(logging_directory / (str(worker_) + '.log'), level='DEBUG', serialize=True)
+        kwargs['logging_level'] = 'DEBUG'
 
         retries = 0
         while retries < 10:
@@ -78,31 +85,18 @@ class ResilientWorker(Worker):
                 sleep(backoff)
         logger.error(f"Ran out of retries. Killing work horse")
 
-    def _monitor_work_horse_tick(self, job):
-        _, ret_val = os.waitpid(self._horse_pid, 0)
-        if ret_val == os.EX_OK:  # The process exited normally.
-            return
-        job_status = job.get_status()
-        if job_status is None:  # Job completed and its ttl has expired
-            return
-        if job_status not in [JobStatus.FINISHED, JobStatus.FAILED]:
-
-            if not job.ended_at:
-                job.ended_at = datetime.datetime.utcnow()
-
-            self.handle_job_failure(job=job)
-
-            # Unhandled failure
-            logger.error(f'(work-horse terminated unexpectedly; waitpid returned {ret_val})')
-
-            retry_handler(job, f"Work-horse process was terminated unexpectedly (waitpid returned {ret_val})",
-                          None, None)
-
-            self.acceptable_failure_count -= 1
-            if self.acceptable_failure_count < 0:
-                logger.error("This worker has had multiple jobs fail. That may mean the host is sick. Terminating "
-                             "the worker to try and shift load onto other hosts.")
-                raise StopRequested()
+    def fork_work_horse(self, job, queue):
+        """Spawns a work horse to perform the actual work and passes it a job.
+        """
+        child_pid = os.fork()
+        os.environ['RQ_WORKER_ID'] = self.name
+        os.environ['RQ_JOB_ID'] = job.id
+        if child_pid == 0:
+            self.main_work_horse(job, queue)
+            os._exit(0)  # just in case
+        else:
+            self._horse_pid = child_pid
+            self.procline('Forked {0} at {1}'.format(child_pid, time()))
 
 
 def worker(parameters: Mapping):
@@ -186,7 +180,7 @@ def worker(parameters: Mapping):
         logger.info(f'Average step length was {exec_time["step_mean_seconds"]:.3f} seconds.')
 
         sim.finalize()
-        metrics = sim.report()
+        metrics = sim.report(print_results=False)
         event['end'] = time()
         end_snapshot = CounterSnapshot()
 
